"use strict";
exports.__esModule = true;
exports.PromiseQueue = void 0;
var assert = require("assert");
var events_1 = require("events");
var PromiseQueue = /** @class */ (function () {
    function PromiseQueue(_a) {
        var _b = _a === void 0 ? {} : _a, _c = _b.concurrency, concurrency = _c === void 0 ? 1 : _c;
        var _this = this;
        /** Number of tasks currently in the queue and awaiting completion. */
        this.size = 0;
        /** Flag indicating the queue is currently processing tasks. */
        this.paused = false;
        /** Number of tasks currently being executed. */
        this.inflight = 0;
        this.bucketQueue = [];
        /** Internal signalling. Used to trigger queue processing. */
        this.emitter = new events_1.EventEmitter();
        assert(concurrency > 0, "'concurrency' must be greater than 0");
        this.concurrency = concurrency;
        this.emitter.on("new-task", this.processTasks.bind(this));
        this.emitter.on("complete", this.processTasks.bind(this));
        this.emitter.on("resume", function () {
            // The queue may process multiple tasks concurrently upon resuming.
            var availableCapacity = _this.concurrency - _this.inflight;
            for (var i = 0; i < availableCapacity; i++) {
                void _this.processTasks();
            }
        });
    }
    /** Pick and execute the highest priority task. */
    PromiseQueue.prototype.processTasks = function () {
        var _this = this;
        if (this.paused || this.inflight >= this.concurrency || this.size === 0) {
            return;
        }
        // Find the lowest index (ie highest priority) task.
        var priorityIndex = this.bucketQueue.findIndex(function (entries) { return entries.length; });
        assert(priorityIndex >= 0, "Expected the queue to contain entries since the queue size is: ".concat(this.size));
        this.inflight += 1;
        // Pick task from a FIFO queue
        var nextTask = this.bucketQueue[priorityIndex].shift();
        this.size -= 1;
        assert(nextTask, "Expected the item list to contain an entry since findIndex() returned != -1");
        var task = nextTask.task, resolve = nextTask.resolve, reject = nextTask.reject;
        // Intentionally not using the await keyword to prevent TS from generating
        // unnecessarily complex ES3 compatible code.
        task()
            .then(resolve, reject)["catch"](function () { return void 0; })
            .then(function () {
            _this.inflight -= 1;
            _this.emitter.emit("complete");
        });
    };
    /** Pause queue execution */
    PromiseQueue.prototype.pause = function () {
        this.paused = true;
    };
    /** Resume queue execution */
    PromiseQueue.prototype.resume = function () {
        this.paused = false;
        this.emitter.emit("resume");
    };
    /** Whether the task processing is currently paused. */
    PromiseQueue.prototype.isPaused = function () {
        return this.paused;
    };
    /** Returns the number of uncompleted tasks on the queue. */
    PromiseQueue.prototype.getSize = function () {
        return this.size;
    };
    /** Returns the number of tasks that are currently inflight. */
    PromiseQueue.prototype.getInflight = function () {
        return this.inflight;
    };
    /** Add a prioritized task to the queue.
     * Note that a lower priority number means that the task will be prioritized.
     * Ie.: `0` is the highest priority value.
     */
    PromiseQueue.prototype.addTask = function (priority, task) {
        var _a;
        var _this = this;
        assert(priority >= 0, "Priority must be >= 0.");
        // If the priority references a bucket that hasn't been initialized we must
        // initialize all the buckets at indices 0 -> priority.
        if (priority >= this.bucketQueue.length) {
            var fillers = new Array(priority - this.bucketQueue.length + 1)
                .fill(0)
                .map(function () { return []; });
            (_a = this.bucketQueue).push.apply(_a, fillers);
        }
        return new Promise(function (resolve, reject) {
            _this.bucketQueue[priority].push({ task: task, resolve: resolve, reject: reject });
            _this.size += 1;
            _this.emitter.emit("new-task");
        });
    };
    return PromiseQueue;
}());
exports.PromiseQueue = PromiseQueue;
